<?php

namespace Research\Subject\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Research\Subject\Http\Requests\SubjectRequest;
use Research\Subject\Interfaces\SubjectRepositoryInterface;
use Research\Subject\Models\Subject;

/**
 * Resource controller class for subject.
 */
class SubjectResourceController extends BaseController
{

    /**
     * Initialize subject resource controller.
     *
     * @param type SubjectRepositoryInterface $subject
     *
     * @return null
     */
    public function __construct(SubjectRepositoryInterface $subject)
    {
        parent::__construct();
        $this->repository = $subject;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Research\Subject\Repositories\Criteria\SubjectResourceCriteria::class);
    }

    /**
     * Display a list of subject.
     *
     * @return Response
     */
    public function index(SubjectRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Research\Subject\Repositories\Presenter\SubjectPresenter::class)
                ->$function();
        }

        $subjects = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('subject::subject.names'))
            ->view('subject::subject.index', true)
            ->data(compact('subjects', 'view'))
            ->output();
    }

    /**
     * Display subject.
     *
     * @param Request $request
     * @param Model   $subject
     *
     * @return Response
     */
    public function show(SubjectRequest $request, Subject $subject)
    {

        if ($subject->exists) {
            $view = 'subject::subject.show';
        } else {
            $view = 'subject::subject.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('subject::subject.name'))
            ->data(compact('subject'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new subject.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SubjectRequest $request)
    {

        $subject = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('subject::subject.name')) 
            ->view('subject::subject.create', true) 
            ->data(compact('subject'))
            ->output();
    }

    /**
     * Create new subject.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SubjectRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $subject                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('subject::subject.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('subject/subject/' . $subject->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/subject/subject'))
                ->redirect();
        }

    }

    /**
     * Show subject for editing.
     *
     * @param Request $request
     * @param Model   $subject
     *
     * @return Response
     */
    public function edit(SubjectRequest $request, Subject $subject)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('subject::subject.name'))
            ->view('subject::subject.edit', true)
            ->data(compact('subject'))
            ->output();
    }

    /**
     * Update the subject.
     *
     * @param Request $request
     * @param Model   $subject
     *
     * @return Response
     */
    public function update(SubjectRequest $request, Subject $subject)
    {
        try {
            $attributes = $request->all();

            $subject->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('subject::subject.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('subject/subject/' . $subject->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('subject/subject/' . $subject->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the subject.
     *
     * @param Model   $subject
     *
     * @return Response
     */
    public function destroy(SubjectRequest $request, Subject $subject)
    {
        try {

            $subject->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('subject::subject.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('subject/subject/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('subject/subject/' . $subject->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple subject.
     *
     * @param Model   $subject
     *
     * @return Response
     */
    public function delete(SubjectRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('subject::subject.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('subject/subject'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/subject/subject'))
                ->redirect();
        }

    }

    /**
     * Restore deleted subjects.
     *
     * @param Model   $subject
     *
     * @return Response
     */
    public function restore(SubjectRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('subject::subject.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/subject/subject'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/subject/subject/'))
                ->redirect();
        }

    }

}
